'* xPL DawnDusk Service
'*
'* Copyright (C) 2003 John Bent
'* http://www.xpl.myby.co.uk/
'*
'* This program is free software; you can redistribute it and/or
'* modify it under the terms of the GNU General Public License
'* as published by the Free Software Foundation; either version 2
'* of the License, or (at your option) any later version.
'* 
'* This program is distributed in the hope that it will be useful,
'* but WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'* GNU General Public License for more details.
'*
'* You should have received a copy of the GNU General Public License
'* along with this program; if not, write to the Free Software
'* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
Option Strict On

Imports System.ServiceProcess

Public Class xplDawnDusk
    Inherits System.ServiceProcess.ServiceBase

    Private WithEvents myXplListener As xpllib.XplListener
    Private mySunrise As sunrise
    Private statusIsDay As Boolean
    Private Today As Date
    Private TodayDawn As Date, TodayDusk As Date


#Region " Component Designer generated code "

    Public Sub New()
        MyBase.New()

        ' This call is required by the Component Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call

    End Sub

    'UserService overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    ' The main entry point for the process
    <MTAThread()> _
    Shared Sub Main()
        Dim ServicesToRun() As System.ServiceProcess.ServiceBase

        ' More than one NT Service may run within the same process. To add
        ' another service to this process, change the following line to
        ' create a second service object. For example,
        '
        '   ServicesToRun = New System.ServiceProcess.ServiceBase () {New Service1, New MySecondUserService}
        '
        ServicesToRun = New System.ServiceProcess.ServiceBase() {New xplDawnDusk}

        System.ServiceProcess.ServiceBase.Run(ServicesToRun)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    ' NOTE: The following procedure is required by the Component Designer
    ' It can be modified using the Component Designer.  
    ' Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container
        Me.ServiceName = "Service1"
    End Sub

#End Region

    Protected Overrides Sub OnStart(ByVal args() As String)
        InitListener()
        ' Setup the sunrise class
        mySunrise = New sunrise
        GetSunInfo()
        statusIsDay = False
        If DateTime.Now().ToUniversalTime() > TodayDawn And DateTime.Now().ToUniversalTime < TodayDusk Then
            statusIsDay = True
        End If
    End Sub

    Protected Overrides Sub OnStop()
        mySunrise = Nothing
        myXplListener = Nothing
    End Sub

    Public Function myXplListener_OnHBeatItems() As String
        Dim s As String
        s = "schema=dawndusk.basic" & Chr(10)
        s = s & "Type=DAYNIGHT" & Chr(10)
        Select Case statusIsDay
            Case True ' day
                s = s & "Status=DAY" & Chr(10)
            Case False ' night
                s = s & "Status=NIGHT" & Chr(10)
        End Select
        Return (s)
    End Function

    Private Sub InitListener()
        Try
            myXplListener = New xpllib.XplListener("johnb-dawndusk", 1)
            myXplListener.ConfigItems.Add("latitude", "53.82", xplConfigTypes.xReconf)
            myXplListener.ConfigItems.Add("longitude", "3.041", xplConfigTypes.xReconf)
            If myXplListener.AwaitingConfiguration Then
                myXplListener.Filters.Add(New XplListener.XplFilter(xpllib.XplMessageTypes.Command, "*", "*", "*", "dawndusk", "request"))
            End If
            myXplListener.XplHBeatItems = AddressOf myXplListener_OnHBeatItems
            myXplListener.XplOnTimer = AddressOf myXpl_OnTimer
            myXplListener.Listen()
        Catch ex As Exception
            If Not EventLog Is Nothing Then
                EventLog.WriteEntry("xPL initialisation failed: " & ex.ToString())
            End If
        End Try
    End Sub

    Private Sub GetSunInfo()
        Try
            mySunrise.Latitude = CDbl(myXplListener.ConfigItems.Item("latitude"))
            mySunrise.Longitude = CDbl(myXplListener.ConfigItems.Item("longitude"))
            Today = DateTime.Now()
            mySunrise.DateDay = Today
            mySunrise.CalculateSun()
            TodayDawn = mySunrise.Sunrise
            TodayDusk = mySunrise.Sunset
        Catch ex As Exception
            EventLog.WriteEntry("Error calculating dawn/dusk times: " & ex.ToString)
        End Try
    End Sub

    Public Sub myXpl_OnTimer()
        ' If the date has changed, recalculate sun info
        If DateTime.Now().ToUniversalTime.Today > Today Then
            GetSunInfo()
        End If

        ' See if dawn has passed
        If Not statusIsDay And DateTime.Now().ToUniversalTime > TodayDawn And DateTime.Now.ToUniversalTime() < TodayDusk Then
            statusIsDay = True
            myXplListener.SendMessage("xpl-trig", "*", "dawndusk.basic", "type=dawndusk" & Chr(10) & "status=dawn" & Chr(10))
        End If

        ' See if dusk has passed
        If statusIsDay And DateTime.Now().ToUniversalTime() > TodayDusk Then
            statusIsDay = False
            myXplListener.SendMessage("xpl-trig", "*", "dawndusk.basic", "type=dawndusk" & Chr(10) & "status=dusk" & Chr(10))
        End If
    End Sub

    Private Sub myXplListener_XplMessageReceived(ByVal sender As Object, ByVal e As xpllib.XplListener.XplEventArgs) Handles myXplListener.XplMessageReceived
        Dim s As String
        Select Case e.XplMsg.GetParam(1, "query").ToLower
            Case "daynight"
                s = "type=daynight" & Chr(10)
                Select Case statusIsDay
                    Case True ' Day
                        s = s & "status=day" & Chr(10)
                    Case False ' Night
                        s = s & "status=night" & Chr(10)
                End Select
                myXplListener.SendMessage("xpl-stat", "*", "dawndusk.basic", s)
            Case "dawndusk"
                s = "type=dawndusk" & Chr(10)
                Select Case statusIsDay
                    Case True ' Day
                        s = s & "status=dawn" & Chr(10)
                    Case False ' Night
                        s = s & "status=dusk" & Chr(10)
                End Select
                myXplListener.SendMessage("xpl-stat", "*", "dawndusk.basic", s)
        End Select
    End Sub

End Class
